%% Reliability analysis of power systems
%  Hybrid version 2, based on the OPAL prototype (analytical), and Pseudo
%  Sequential Monte Carlo Simulation
%  Written by Trygve V. Berg as part of a Master thesis

%% Initializing test network from Excel file
%  RBTS 6bus is used while constructing this code

tic; %starting clock for runtime

% Location of Excel file for input of case data and output of results
inputfile = 'Case files\RBTS_benchmark.xls';

% Load MATPOWER case data (network data)
mpc = read_mpc_from_Excel(inputfile);

% Load settings/options for the contingency analysis
contanalysis_settings = read_contanalysis_settings_from_Excel(inputfile);

% Load input data for corrective actions
corrective_actions_data = read_corrective_actions_data_from_Excel(inputfile);

% Setting up MATPOWER options
mpopt = setup_mpopt_opalcons(contanalysis_settings);

% Load input data for operating states
os_data = read_os_data_from_Excel(inputfile);

% Load settings/options for constructing the contingency/outage list
ol_settings = read_ol_settings_from_Excel(inputfile);

% Read delivery point data
[dp_data, ~] = read_loaddata_from_Excel(inputfile);
dp_buses = dp_data.bus;

%% OPAL prototype evaluation of first order contingencies

%Only evaluating the 1st order line outages
ol_settings.branch_outage_order = 1; 

% Construct the contingency/outage list 
ol = create_ol(ol_settings,mpc);

% Run contingency analysis
cont_results = opalcons(mpc, ol, contanalysis_settings, corrective_actions_data, os_data, dp_buses);

% Load general reliability analysis settings from Excel file
reliabilitycalc_settings = read_reliabilitycalc_settings_from_Excel(inputfile);

% Read reliability/outage data
rel_data = read_rel_data_from_Excel(inputfile,reliabilitycalc_settings);

% Read component reliability statistics data
componentdata = read_componentdata(reliabilitycalc_settings.component_database,0,[]);

% Update reliability data with component reliability statistics where relevant
rel_data.outagedata = update_outage_data_from_stats(rel_data.outagedata,componentdata);

% Read delivery point data
[dp_data, ~] = read_loaddata_from_Excel(inputfile);

% Setting up convenient data structure for interruption cost function calculations
customer_database = reliabilitycalc_settings.customer_database;
[intcost_func, dp_data] = create_intcost_func(customer_database,...
    reliabilitycalc_settings,os_data,dp_data);

% Run reliability analysis
year = 1;
[aggres, cutres, cut_idx] = reliabilitycalc(reliabilitycalc_settings,...
    year,cont_results,dp_data,componentdata,rel_data,os_data,intcost_func);


%% Pseudo sequential MCS of higher order contingencies

% Resetting the line outage settings to include higher orders
ol_settings.branch_outage_order = 2; 

% Creating a matrix with line data
%          Line nb   Fbus    Tbus      FOR       Lambda    Mu   Curr S  Next S            
linedata = [ 1        1       3      15/8736     1.717E-4  0.1    0       0;
             2        2	      4	     50/8736     5.7234E-4 0.1    0       0;
             3        1	      2	     40/8736     4.5788E-4 0.1    0       0;
             4        3	      4	     10/8736     1.1447E-4 0.1    0       0;
             5        3	      5	     10/8736     1.1447E-4 0.1    0       0;
             6        1	      3	     15/8736     1.717E-4  0.1    0       0;
             7        2	      4	     50/8736     5.7234E-4 0.1    0       0;
             8        4	      5	     10/8736     1.1447E-4 0.1    0       0;
             9        5	      6	     10/8736     1.1447E-4 0.1    0       0];
         
%FOR =(faults per year * repair time )/ hours in a year
% State = 1 for down, and 0 for up


% Simulation start

N = 15000 ; %number of simulation years
M = 8736; %hours in a year
L = length(linedata(:,1));%number of lines

lambdadp_a = zeros(length(dp_buses),N); %annual frequency of interruption
durationdp_a = zeros(length(dp_buses),N); %annual duration of interruptions
Pinterrdp_a = zeros(length(dp_buses),N); % annual interrupted power for each dp
EENSdp_a = zeros(length(dp_buses),N);%yearly EENS for each delivery point 

for y = 1:N %loop through all sim years
    Ploss = 0; %loss of load due to curtailment
    
    outagecounter = zeros(length(dp_buses),1); %number of outages for each dp for a specific year
    lambdadp_y = zeros(length(dp_buses),1); %specific year frequency of interruption for each dp
    durationdp_y = zeros(length(dp_buses),1); %specific year duration of interruptions for each dp
    Pinterrdp_y = zeros(length(dp_buses),1); %specific year interrupted power for each dp
    EENSdp_y = zeros(length(dp_buses),1);% sum of EENS for each dp for this specific year

    
    
    for i = 1:M %loop through a year in hours
        EENSdp_h = 0; %energy not supplied for the specific contingency in hourly increment
        outagebranches = []; %lines that are on outage
        index = 1; %index used for storing the outage lines in order
        linedata(:,8) = 0;
        for k = 1:L %loop through all lines
            U = rand(); %random uniform number
            if U < linedata(k,4) % line outage at hourly increment
                outagebranches(index) = k; %adding specific line to the outage vector
                linedata(k,8) = 1; % adding the outages to the state column
                index = index+1;
            end %if U
        end %k
        if length(outagebranches) >= 2
            if i < 2184 % if we are within the first 3 months (in hours) use "heavy load" operating state
                i_state = 1;
            else % we are in any of the other months (in hours) use "light load" operating state
                i_state = 2;
            end %if i <8736
            mpc = init_os(mpc,os_data,i_state); %Initialize the correct os for use in contingency analysis
            Ploss = contanalysis_singlecont_montecarlosim(outagebranches,mpc,dp_buses,mpopt,corrective_actions_data,contanalysis_settings); %consequence analysis of sampled lines
            if sum(Ploss) > 0 %if outage causes power loss, do forward and backward simulation
                
                %Forward simulation
                [durationforw,outagecounterforw,Plossforw,EENSforw] = forwardsim(linedata,L,outagebranches,mpc,dp_buses,mpopt,corrective_actions_data,contanalysis_settings);
                
                %Backward simulation
                [durationbackw,outagecounterbackw,Plossbackw,EENSbackw] = backwardsim(linedata,L,outagebranches,mpc,dp_buses,mpopt,corrective_actions_data,contanalysis_settings);
                
                %Accumulate indices from both forward and backward
                %simulation
                durationdp_y = durationdp_y + durationforw + durationbackw; % accumulating outage duration for the year
                outagecounter = outagecounter + outagecounterforw + outagecounterbackw; % accumulating number of ouatages for the year   
                Pinterrdp_y = Pinterrdp_y + Plossforw + Plossbackw; % accumulating interrupted power for the year
                EENSdp_y = EENSdp_y + EENSforw + EENSbackw; % adding hourly EENS to total EENS for the year
               
            end %if Ploss
        end %if length
    end %i
    for p = 1:length(dp_buses)
        lambdadp_a(p,y) = outagecounter(p)/8736; %calculating and adding number this years interruptions to annual number of intterruptions matrix
        durationdp_a(p,y) = durationdp_y(p); %adding this years outage duration to the annual outage duration matrix
        Pinterrdp_a(p,y) = Pinterrdp_y(p); %adding the interrupted power from the specific year to the annual Pinterr matrix
        EENSdp_a(p,y) = EENSdp_y(p); %adding the EENS from this year to annual EENS matrix
    end %for p
end %y

%% Reliability analysis
%  Combining the results from OPAL and MCS 

% Calculating indices for each dp
Pinterrdp_a_avg = (sum(Pinterrdp_a,2))/N + aggres.sum_dp.Pinterr_a/1000; %dividing the reliabilitycalc part by 1000 because it is in kW (as opposed to MW)
EENSdp_a_avg = (sum(EENSdp_a,2))/N + aggres.sum_dp.ENS_a/1000; 
lambdadp_a_avg = (sum(lambdadp_a,2))/N + aggres.sum_dp.lambda_a;
durationdp_a_avg = (sum(durationdp_a,2))/N + aggres.sum_dp.U_a;

%Calculating system indices
Pinterrsys_a_avg = sum(sum(Pinterrdp_a))/N + aggres.sum_sys.Pinterr_a/1000;
EENSsys_a_avg = sum(sum(EENSdp_a))/N + aggres.sum_sys.ENS_a/1000;
lambdasys_a_avg = sum(sum(lambdadp_a))/N + sum(aggres.sum_dp.lambda_a);
durationsys_a_avg = sum(sum(durationdp_a))/N + sum(aggres.sum_dp.U_a);

% Calculating the standard deviation and CV for the MCS part
EENSsys_tot = sum(sum(EENSdp_a)); %total EENS for all years
EENSsys_avg = sum(EENSsys_tot)/N; % average EENS over N simulation years
EENSsys_a = sum(EENSdp_a); % annual EENS for N sim years
expression = 0;
for o = 1:N
    expression = expression + (EENSsys_a(o)-EENSsys_avg)^2; %summing up parts of standard deviation
end
EENSsys_stdd = sqrt(((1/(N-1))*expression)); %standard deviation
EENSsys_cv = EENSsys_stdd/(sqrt(N)*EENSsys_avg); %coefficient of variation

% Storing all indices in one struct
indices = {}; 
indices.dp = {}; 
indices.sys = {};
[indices.dp.Pinterrdp_a_avg,indices.dp.EENSdp_a_avg,indices.dp.lambdadp_a_avg,indices.dp.durationdp_a_avg] = deal(Pinterrdp_a_avg,EENSdp_a_avg,lambdadp_a_avg,durationdp_a_avg);
[indices.sys.Pinterrsys_a_avg,indices.sys.EENSsys_a_avg,indices.sys.lambdasys_a_avg,indices.sys.durationsys_a_avg,indices.sys.EENSsys_cv] = deal(Pinterrsys_a_avg,EENSsys_a_avg,lambdasys_a_avg,durationsys_a_avg,EENSsys_cv);


toc;