%% Reliability analysis of power systems
%  Benchmark OPAL on 4-area network
%  Written by Trygve V. Berg as part of a Master thesis

%% Initializing test network from Excel file
%  SAMREL 4-area network

tic; %starting clock for runtime

% Location of Excel file for input of case data and output of results
inputfile = 'Case files\4-area_network_extra_line.xls';

% Load MATPOWER case data (network data)
mpc = read_mpc_from_Excel(inputfile);

% Load settings/options for the contingency analysis
contanalysis_settings = read_contanalysis_settings_from_Excel(inputfile);

% Load input data for corrective actions
corrective_actions_data = read_corrective_actions_data_from_Excel(inputfile);

% Setting up MATPOWER options
mpopt = setup_mpopt_opalcons(contanalysis_settings);

% Load input data for operating states
os_data = read_os_data_from_Excel(inputfile);

% Load settings/options for constructing the contingency/outage list
ol_settings = read_ol_settings_from_Excel(inputfile);

% Read delivery point data
[dp_data, ~] = read_loaddata_from_Excel(inputfile);
dp_buses = dp_data.bus;

%% OPAL prototype evaluation of first order contingencies

%Only evaluating the 1st order line outages
ol_settings.branch_outage_order = 2; 

% Construct the contingency/outage list 
ol = create_ol(ol_settings,mpc);

% Run contingency analysis
cont_results = opalcons(mpc, ol, contanalysis_settings, corrective_actions_data, os_data, dp_buses);

% Load general reliability analysis settings from Excel file
reliabilitycalc_settings = read_reliabilitycalc_settings_from_Excel(inputfile);

% Read reliability/outage data
rel_data = read_rel_data_from_Excel(inputfile,reliabilitycalc_settings);

% Read component reliability statistics data
componentdata = read_componentdata(reliabilitycalc_settings.component_database,0,[]);

% Update reliability data with component reliability statistics where relevant
rel_data.outagedata = update_outage_data_from_stats(rel_data.outagedata,componentdata);

% Read delivery point data
[dp_data, ~] = read_loaddata_from_Excel(inputfile);

% Setting up convenient data structure for interruption cost function calculations
customer_database = reliabilitycalc_settings.customer_database;
[intcost_func, dp_data] = create_intcost_func(customer_database,...
    reliabilitycalc_settings,os_data,dp_data);

% Run reliability analysis
year = 1;
[aggres, cutres, cut_idx] = reliabilitycalc(reliabilitycalc_settings,...
    year,cont_results,dp_data,componentdata,rel_data,os_data,intcost_func);


%% Reliability analysis
%  Combining the results from OPAL 

% Calculating indices for each dp
Pinterrdp_a_avg = aggres.sum_dp.Pinterr_a/1000; %dividing the reliabilitycalc part by 1000 because it is in kW (as opposed to MW)
EENSdp_a_avg = aggres.sum_dp.ENS_a/1000; 
lambdadp_a_avg = aggres.sum_dp.lambda_a;
durationdp_a_avg = aggres.sum_dp.U_a;

%Calculating system indices
Pinterrsys_a_avg = aggres.sum_sys.Pinterr_a/1000;
EENSsys_a_avg = aggres.sum_sys.ENS_a/1000;
lambdasys_a_avg = sum(aggres.sum_dp.lambda_a);
durationsys_a_avg = sum(aggres.sum_dp.U_a);

% Storing all indices in one struct
indices = {}; 
indices.dp = {}; 
indices.sys = {};
[indices.dp.Pinterrdp_a_avg,indices.dp.EENSdp_a_avg,indices.dp.lambdadp_a_avg,indices.dp.durationdp_a_avg] = deal(Pinterrdp_a_avg,EENSdp_a_avg,lambdadp_a_avg,durationdp_a_avg);
[indices.sys.Pinterrsys_a_avg,indices.sys.EENSsys_a_avg,indices.sys.lambdasys_a_avg,indices.sys.durationsys_a_avg] = deal(Pinterrsys_a_avg,EENSsys_a_avg,lambdasys_a_avg,durationsys_a_avg);


toc;

